/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyright (C) 2008 Ehsan Akhgari <ehsan.akhgari+forcertl@gmail.com>,
 *                    Kai Liu <kliu@mozilla.kailiu.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

Components.utils.import("resource://forcertl/forcertl.js");

const forcertl_stylesheet_url = "chrome://forcertl/content/intl.css";
const forcertl_pref = "extensions.forcertl.rtl";
const forcertl_localedir_pref = "intl.uidirection." + navigator.language;

function forcertl_in_rtl_mode()
{
  return forcertl_data.rtl;
}

function forcertl_switch_mode()
{
  if (forcertl_have_api_support() &&
      forcertl_switch_stylesheet() &&
      forcertl_switch_chromedir()) {
    forcertl_data.rtl = !forcertl_data.rtl;
    nsPreferences.setBoolPref(forcertl_pref, forcertl_data.rtl);
    if (forcertl_data.mozlocaledir_supported) {
      nsPreferences.mPrefService.setCharPref(forcertl_localedir_pref,
                                             forcertl_data.rtl ? "rtl" : "ltr");
    }
  }
}

function forcertl_have_api_support()
{
  return ("@mozilla.org/content/style-sheet-service;1" in Components.classes) &&
         (!Components.ID('{41d979dc-ea03-4235-86ff-1e3c090c5630}')
                     .equals(Components.interfaces.nsIStyleSheetService));
}

function forcertl_switch_stylesheet()
{
  var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"]
                      .getService(Components.interfaces.nsIStyleSheetService);
  var ios = Components.classes["@mozilla.org/network/io-service;1"]
                      .getService(Components.interfaces.nsIIOService);
  var uri = ios.newURI(forcertl_stylesheet_url, null, null);
  if (forcertl_data.rtl) {
    if (sss.sheetRegistered(uri, sss.AGENT_SHEET)) {
      sss.unregisterSheet(uri, sss.AGENT_SHEET);
      return !sss.sheetRegistered(uri, sss.AGENT_SHEET);
    }
  } else {
    if (!sss.sheetRegistered(uri, sss.AGENT_SHEET)) {
      sss.loadAndRegisterSheet(uri, sss.AGENT_SHEET);
      return sss.sheetRegistered(uri, sss.AGENT_SHEET);
    }
  }
  return false;
}

function forcertl_switch_chromedir()
{
  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                     .getService(Components.interfaces.nsIWindowMediator);
  var enumerator = wm.getEnumerator(null);
  while(enumerator.hasMoreElements()) {
    var win = enumerator.getNext();
    forcertl_change_chromedir_in_document(win.document,
                                          forcertl_data.rtl ? "ltr" : "rtl");
  }

  if (!forcertl_data.observer_obj) {
    var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                       .getService(Components.interfaces.nsIWindowWatcher);
    forcertl_data.observer_obj = new forcertl_observer();
    ww.registerNotification(forcertl_data.observer_obj);
  }

  return true;
}

function forcertl_change_chromedir_in_document(doc, dir)
{
  if (!(doc instanceof Components.interfaces.nsIDOMXULDocument) &&
      !(doc instanceof Components.interfaces.nsIDOMHTMLDocument)) {
    try {
      doc = doc.QueryInterface(Components.interfaces.nsIDOMXULDocument);
    } catch(e) {
      try {
        doc = doc.QueryInterface(Components.interfaces.nsIDOMHTMLDocument);
      } catch (e) {}
    }
  }
  if (doc instanceof Components.interfaces.nsIDOMXULDocument ||
      doc instanceof Components.interfaces.nsIDOMHTMLDocument) {

    // Try to limit ourselves to UI-related documents
    if (!( doc.location.protocol == "chrome:" ||
           doc.location.protocol == "about:" ||
           doc.baseURI.substr(0, 15) == "about:neterror?" ||
           // XXXkliu:
           // We want to handle local file listings, as that's browser-generated
           // content, but not local files; it seems that checking for a
           // trailing slash in the baseURI is a good heuristic.
           doc.baseURI.search(/^file:.+\/$/) != -1 ||
           // XXXehsan:
           // This is kind of a hack.  Some FTP sites provide index pages, which we should not
           // touch.  Others do not, and Firefox displays its own dirListing page which we need
           // to detect.  I do this by checking for an element with ID "UI_goUp", because I was
           // not able to find a more elegant way.  :(
           (doc.baseURI.search(/^ftp:.+\/$/) != -1 && doc.getElementById("UI_goUp")) ))
    {
      return;
    }

    try {
      var walker = Components.classes["@mozilla.org/inspector/deep-tree-walker;1"]
                             .createInstance(Components.interfaces.inIDeepTreeWalker);
    } catch(e) { return; }

    const SHOW_ELEMENT = Components.interfaces.nsIDOMNodeFilter.SHOW_ELEMENT;
    walker.showAnonymousContent = true;
    walker.showSubDocuments = false;
    walker.init(doc.documentElement, SHOW_ELEMENT);

    if (true) {
      // XXXkliu:
      // Unfortuantely, some anonymous nodes will not show up unless they have
      // been "initialized" (either drawn or walked), which means that some
      // hidden anonymous content will not get updated unless we do this
      // dry-run walk through the tree
      while (walker.nextNode()) {
        // do nothing
      }
      walker = Components.classes["@mozilla.org/inspector/deep-tree-walker;1"]
                         .createInstance(Components.interfaces.inIDeepTreeWalker);
      walker.showAnonymousContent = true;
      walker.showSubDocuments = false;
      walker.init(doc.documentElement, SHOW_ELEMENT);
    }

    var node = walker.currentNode;
    do {
      if (node instanceof Components.interfaces.nsIDOMXULElement) {
        if (node.hasAttribute("chromedir")) node.setAttribute("chromedir", dir);

        // Walk sub-documents
        if (node.localName == "browser")
          forcertl_change_chromedir_in_document(node.contentDocument, dir);
      }
      else if (node instanceof Components.interfaces.nsIDOMHTMLElement) {
        if (node.hasAttribute("dir")) node.setAttribute("dir", dir);
      }
      walker.nextNode();
    } while (node = walker.nextNode());
  }
}

function forcertl_observer()
{
}

forcertl_observer.prototype = {
  observe: function(aSubject, aTopic, aData) {
    if (aTopic == "domwindowopened") {
      try {
        var win = aSubject.QueryInterface(Components.interfaces.nsIDOMWindow);
        win.addEventListener("load", forcertl_new_window_loaded, false);
      } catch (e) {}
    }
  }
};

function forcertl_new_window_loaded(event)
{
  forcertl_change_chromedir_in_document(event.target,
                                        forcertl_data.rtl ? "rtl" : "ltr");
  event.target.addEventListener("DOMContentLoaded", forcertl_new_document_loaded, true);
}

function forcertl_new_document_loaded(event)
{
  forcertl_change_chromedir_in_document(event.originalTarget,
                                        forcertl_data.rtl ? "rtl" : "ltr");
}

function forcertl_init()
{
  if (!forcertl_data.inited) {
    forcertl_data.inited = true;
    new forcertl_uninit_observer();
    if (nsPreferences.getBoolPref(forcertl_pref, false)) {
      // we've been set to RTL from the last session
      forcertl_switch_mode();
    }
  }

  window.addEventListener("DOMContentLoaded", forcertl_new_document_loaded, true);
}

function forcertl_uninit_observer()
{
  this.register();
}

forcertl_uninit_observer.prototype = {
  register: function() {
    var observerService = Components.classes["@mozilla.org/observer-service;1"]
                          .getService(Components.interfaces.nsIObserverService);
    observerService.addObserver(this, "quit-application-granted", false);
  },
  observe: function(aSubject, aTopic, aData) {
    if (aTopic == "quit-application-granted") {
      var observerService = Components.classes["@mozilla.org/observer-service;1"]
                            .getService(Components.interfaces.nsIObserverService);
      observerService.removeObserver(this, "quit-application-granted");

      if (forcertl_data.observer_obj) {
        var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                           .getService(Components.interfaces.nsIWindowWatcher);
        ww.unregisterNotification(forcertl_data.observer_obj);
      }
    }
  }
}

window.addEventListener("load", forcertl_init, false);
